use pyo3::{PyAny, PyResult};

pub trait PyAnyExtras<'py> {
    fn qualified_class_name(&self) -> PyResult<String>;
    fn get_proto_meta(&self) -> PyResult<&'py PyAny>;
    fn create_instance(&self, field_name: &str) -> PyResult<&'py PyAny>;
}

impl<'py> PyAnyExtras<'py> for &'py PyAny {
    fn qualified_class_name(&self) -> PyResult<String> {
        let class = self.getattr("__class__")?;
        let module = class.getattr("__module__")?;
        let name = class.getattr("__name__")?;
        Ok(format!("{module}.{name}"))
    }

    fn get_proto_meta(&self) -> PyResult<&'py PyAny> {
        self.getattr("_betterproto")
    }

    fn create_instance(&self, field_name: &str) -> PyResult<&'py PyAny> {
        let res = self.getattr("cls_by_field")?
            .get_item(field_name)?
            .call0()?;
        Ok(res)
    }
}
